@@ -22,12 +22,11 @@ from docutils.utils import Reporter
 from docutils.parsers.rst import directives, roles, convert_directive_function
 
 from sphinx.errors import ExtensionError
-from sphinx.locale import _
+from sphinx.locale import __
 from sphinx.util import logging
 
 logger = logging.getLogger(__name__)
-report_re = re.compile('^(.+?:(?:\\d+)?): \\((DEBUG|INFO|WARNING|ERROR|SEVERE)/(\\d+)?\\) '
-                       '(.+?)\n?$')
+report_re = re.compile('^(.+?:(?:\\d+)?): \\((DEBUG|INFO|WARNING|ERROR|SEVERE)/(\\d+)?\\) ')
 
 if False:
     # For type annotation
@@ -162,7 +161,8 @@ class WarningStream(object):
         if not matched:
             logger.warning(text.rstrip("\r\n"))
         else:
-            location, type, level, message = matched.groups()
+            location, type, level = matched.groups()
+            message = report_re.sub('', text).rstrip()  # type: ignore
             logger.log(type, message, location=location)
 
 
@@ -193,6 +193,6 @@ def directive_helper(obj, has_content=None, argument_spec=None, **option_spec):
         return convert_directive_function(obj)
     else:
         if has_content or argument_spec or option_spec:
-            raise ExtensionError(_('when adding directive classes, no '
-                                   'additional arguments may be given'))
+            raise ExtensionError(__('when adding directive classes, no '
+                                    'additional arguments may be given'))
         return obj
